'' =================================================================================================
''
''   File....... read_gates.spin
''   Purpose....
''   Author.....
''   E-mail.....
''   Started....
''   Updated....
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


con { pab io pins }                                               
                                                                  
  RX1     = 31                                                   ' programming / terminal
  TX1     = 30                                                    
                                                                  
  SDA     = 29                                                   ' eeprom / i2c
  SCL     = 28                                                    
                                                                  
  AUD_RT  = 27                                                   ' audio (d/a) outputs
  AUD_LF  = 26                                                    
                                                                  
  LED_27  = 27                                                   ' leds (on audio outputs)
  LED_26  = 26                                                    
                                                                  
  SD_CS   = 25                                                   ' usd card
  SD_DI   = 24                                                    
  SD_CLK  = 23
  SD_DO   = 22

  ADC_CS  = 21                                                   ' adc input
  ADC_CLK = 20                                                    
  ADC_DO  = 19 { miso }                                           
  ADC_DI  = 18 { mosi }                                           
                                                                  
  SVO_6   = 17                                                   ' servo headers
  SVO_5   = 16

  SVO_4   = 15
  SVO_3   = 14

  SVO_R   = 13
  SVO_L   = 12


con

  #0, IS_OFF, IS_ON

  THRESHOLD = 4080


obj

' main                                                           ' * master Spin cog
  time : "jm_time"                                               '   timing/delays
  io   : "jm_io_basic"                                           '   essential io control
  adc  : "jm_adc124s021"                                         '   analog input               
  term : "jm_fullduplexserial"                                   ' * terminal io
' gates                                                          ' * background adc measurement                                 

' * Uses cog when loaded


var

  long  chan0                                                    ' analog channels
  long  chan1
  long  chan2
  long  chan3

  long  millis                                                   ' milliseconds timer

  long  gstack[32]                                               ' stack for gates cog


pub main | elapsed

  setup

  repeat
    repeat until (chan0 < THRESHOLD)
    elapsed := -millis                                           ' start timing
    outa[LED_26] := IS_ON
     
    repeat until (chan2 < THRESHOLD) 
    elapsed += millis                                            ' end timing
    elapsed := (elapsed + 5) / 10                                ' round up to 1/100ths
    outa[LED_26] := IS_OFF

    term.str(string("Elapsed time: "))                           ' report 
    term.dec(elapsed / 100)
    term.tx(".")
    term.rjdec(elapsed // 100, 2, "0")
    term.tx(term#CR)


pub setup

'' Setup IO and objects for application

  time.start                                                     ' setup timing/delays for clock speed

  io.start(0, 0)                                                 ' clear all pins
  dira[LED_27..LED_26] := %11                                    ' make LEDs outputs

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal

  cognew(gates, @gstack)                                         ' start background ADC

  time.pause(10)                                                 ' allow ADC readings before starting timer


pri gates | t, ch

'' Scan analog channels every millisecond
'' -- update global millis value

  adc.start(ADC_CS, ADC_CLK, ADC_DI, ADC_DO)                     ' start adc

  t := cnt                                                       ' sync time point
  repeat
    repeat ch from 0 to 3                                        ' loop through channels
      long[@chan0][ch] := adc.read(ch)                           ' read adc
      waitcnt(t += MS_001)                                       ' allow 1ms window to expire
      ++millis                                                   ' update millis register


dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}